@@ -2,18 +2,24 @@ package Catalyst::Base;
 
 use strict;
 use base qw/Class::Data::Inheritable Class::Accessor::Fast/;
+
+use Catalyst::Exception;
 use NEXT;
 
-__PACKAGE__->mk_classdata($_) for qw/_cache _config/;
-__PACKAGE__->_cache( [] );
+__PACKAGE__->mk_classdata($_) for qw/_attr_cache _action_cache _config/;
+__PACKAGE__->_attr_cache( {} );
+__PACKAGE__->_action_cache( [] );
 
 # note - see attributes(3pm)
 sub MODIFY_CODE_ATTRIBUTES {
     my ( $class, $code, @attrs ) = @_;
-    push @{ $class->_cache }, [ $code, [@attrs] ];
+    $class->_attr_cache->{$code} = [@attrs];
+    push @{ $class->_action_cache }, [ $code, [@attrs] ];
     return ();
 }
 
+sub FETCH_CODE_ATTRIBUTES { $_[0]->_attr_cache->{ $_[1] } || () }
+
 =head1 NAME
 
 Catalyst::Base - Catalyst Universal Base Class
@@ -65,7 +71,11 @@ component loader with config() support and a process() method placeholder.
 
 sub new {
     my ( $self, $c ) = @_;
-    return $self->NEXT::new( $self->config );
+ 
+    # Temporary fix, some components does not pass context to constructor
+    my $arguments = ( ref( $_[-1] ) eq 'HASH' ) ? $_[-1] : {};
+
+    return $self->NEXT::new( { %{ $self->config }, %{ $arguments } } );
 }
 
 # remember to leave blank lines between the consecutive =item's
@@ -82,8 +92,8 @@ sub new {
 sub config {
     my $self = shift;
     $self->_config( {} ) unless $self->_config;
-    if ( $_[0] ) {
-        my $config = $_[1] ? {@_} : $_[0];
+    if (@_) {
+        my $config = @_ > 1 ? {@_} : $_[0];
         while ( my ( $key, $val ) = each %$config ) {
             $self->_config->{$key} = $val;
         }
@@ -95,7 +105,16 @@ sub config {
 
 =cut
 
-sub process { die ((ref $_[0] || $_[0])." did not override Catalyst::Base::process"); }
+sub process {
+
+    Catalyst::Exception->throw( 
+        message => ( ref $_[0] || $_[0] ) . " did not override Catalyst::Base::process"
+    );
+}
+
+=item FETCH_CODE_ATTRIBUTES
+
+=item MODIFY_CODE_ATTRIBUTES
 
 =back
 
